/*
** jRes.c: jRes main file.
** Copyright (C) 1997-1998, Nullsoft Inc.
** Free for noncommercial use.
** http://www.nullsoft.com/
**/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shellapi.h>
#include "resource.h"

#include <stdlib.h>

static char szAppName[] = "jRes";
HWND hMainWindow;
HINSTANCE hMainInstance;
void systray_minimize(char *tip);
void systray_restore();

static HMENU main_menu;

typedef struct _tagModeType {
	int xres, yres, bits, ref;
} ModeType;

int num_color_depths;
int *color_depths;
int num_resolutions;
int *resolutions;
int cur_res=0, cur_depth=0;
int ModeList_Length;
ModeType *ModeList;
void MakeModeList();
void MakeMenu(HMENU hmenu);
void setTip();

int SetVidMode(int xres, int yres, int bpp, int ref);
void doVidModeErr(int m);


LRESULT CALLBACK WndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam);

BOOL Register(HINSTANCE hInst)
{
  WNDCLASSEX WndClassEx;

  hMainInstance = hInst;

  WndClassEx.cbSize        = sizeof(WNDCLASSEX);
  WndClassEx.style         = 0;
  WndClassEx.lpfnWndProc   = WndProc;
  WndClassEx.cbClsExtra    = 0;
  WndClassEx.cbWndExtra    = 0;
  WndClassEx.hInstance     = hInst;
  WndClassEx.hIcon         = 0;
  WndClassEx.hCursor       = 0;
  WndClassEx.hbrBackground = NULL;
  WndClassEx.lpszMenuName  = NULL;
  WndClassEx.lpszClassName = szAppName;
  WndClassEx.hIconSm	   = 0;

  if (!RegisterClassEx(&WndClassEx)) return FALSE;
  else return TRUE;
}

HWND CreateJRes(HINSTANCE hInst)
{
	DWORD dwStyle = 0;
	DWORD dwStyleEx = 0;
	HWND hwnd;

	hwnd = CreateWindowEx(dwStyleEx,szAppName, szAppName, dwStyle, 
						 0,0,0,0, NULL,NULL,hInst,NULL);
	return hwnd;
}


int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInstance, LPSTR lpszCmdParam, int nCmdShow)
{
	MSG Msg;


	if (FindWindow(szAppName,szAppName)) return TRUE;


	MakeModeList();
    main_menu = LoadMenu(hMainInstance,MAKEINTRESOURCE(IDR_MENU1));
    main_menu = GetSubMenu(main_menu,0);
	MakeMenu(main_menu);
	free(ModeList);
	{
		int x = GetSystemMetrics(SM_CXSCREEN);
		int y = GetSystemMetrics(SM_CYSCREEN);
		for (cur_res = 0; cur_res < num_resolutions; cur_res ++)
		{
			if (resolutions[cur_res*3] == x && resolutions[cur_res*3+1] == y)
				break;
		}
		if (cur_res == num_resolutions) cur_res--;

	}
	{
		HDC hdc = GetDC(GetDesktopWindow());
		int c= GetDeviceCaps(hdc,BITSPIXEL);
		ReleaseDC(GetDesktopWindow(),hdc);
		for (cur_depth = 0; cur_depth < num_color_depths; cur_depth ++)
		{
			if (color_depths[cur_depth] == c)
				break;
		}
		if (cur_depth == num_color_depths) cur_depth--;
	}
	CheckMenuItem(main_menu,cur_depth+ID_BPPBASE,MF_CHECKED);
	CheckMenuItem(main_menu,cur_res+ID_MODEBASE,MF_CHECKED);

	if (!Register(hInst))
	{
		MessageBox(NULL,"Couldn't Register Window Class","Error",MB_OK|MB_ICONSTOP);
		return 1;
	}
	if (!CreateJRes(hInst))
	{
		MessageBox(NULL,"Couldn't Create Window","Error",MB_OK|MB_ICONSTOP);
		return 1;
	}
	setTip();

	while (GetMessage(&Msg,NULL,0,0))
	{
		TranslateMessage(&Msg);
		DispatchMessage(&Msg);
	}
	return 0;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	int x, a;
   switch(Message)
   {
      case WM_CREATE:
		  hMainWindow = hwnd;
		  systray_minimize("jRes");
      return TRUE;
	  case WM_COMMAND:
		  switch (LOWORD(wParam))
		  {
			case IDOK: DestroyWindow(hwnd); break;
			case ID_ABOUT: MessageBox(hwnd,
							   "jRes v0.6 - A handy display adjustment program\n\n"
							   "Copyright (C) 1997, Nullsoft\n"
							   "Code by Justin Frankel, Icon by Nicholas Head\n"
							   "Freeware\n"
							   "http://www.nullsoft.com/\n"
							   "(Greets to #winprog!)",
							   "About jRes",MB_OK); 
				return FALSE;
			case ID_DISPOPT:
				ShellExecute(hwnd,"open","control","desk.cpl","",SW_SHOWNORMAL);
			return FALSE;
			default:
				x = LOWORD(wParam);		
				if (x >= ID_MODEBASE && x <= ID_MODEBASE + num_resolutions)
				{	
					int fa = -666;
					int t=cur_depth;
					x = x - ID_MODEBASE;
					do
					{
						a = SetVidMode(resolutions[x*3],resolutions[x*3+1],color_depths[t],resolutions[x*3+2]);
						if (fa == -666) fa = a;
						if (a == DISP_CHANGE_NOTUPDATED || a == DISP_CHANGE_RESTART)
						{
							t=-1;
							break;
						}
						if (a == DISP_CHANGE_SUCCESSFUL)
						{
						}
						else t--;
					} while (a != DISP_CHANGE_SUCCESSFUL && t >= 0);
					if (t < 0) 
					{
						doVidModeErr(fa);
					}
				}
				else if (x >= ID_BPPBASE && x <= ID_BPPBASE + num_color_depths)
				{					
					int fa=-666;
					int t = cur_res;
					x = x - ID_BPPBASE;
					do
					{
						a = SetVidMode(resolutions[t*3],resolutions[t*3+1],color_depths[x],
							resolutions[t*3+2]);
						if (fa == -666) fa = a;
						if (a == DISP_CHANGE_NOTUPDATED || a == DISP_CHANGE_RESTART)
						{
							t=-1;
							break;
						}
						if (a == DISP_CHANGE_SUCCESSFUL)
						{
						}
						else t--;
					} while (a != DISP_CHANGE_SUCCESSFUL && t >= 0);
					if (t < 0) 
					{
						doVidModeErr(fa);
					}
				}
		  }
	  return FALSE;
	  case WM_DISPLAYCHANGE:
		{
			int x;
			CheckMenuItem(main_menu,cur_depth+ID_BPPBASE,MF_UNCHECKED);
			CheckMenuItem(main_menu,cur_res+ID_MODEBASE,MF_UNCHECKED);
			for (x = 0; x < num_color_depths; x ++)
				if (color_depths[x] == (int)wParam)
					break;
			if (x >= num_resolutions) x--;
			cur_depth = (int) x;
			for (x = 0; x < num_resolutions; x ++)
				if (resolutions[x*3] == LOWORD(lParam) &&
					resolutions[x*3+1] == HIWORD(lParam))
					break;
			if (x >= num_resolutions) x--;
			cur_res = x;
			CheckMenuItem(main_menu,cur_depth+ID_BPPBASE,MF_CHECKED);
			CheckMenuItem(main_menu,cur_res+ID_MODEBASE,MF_CHECKED);
			setTip();
		}
	  return FALSE;
      case WM_CLOSE:
         DestroyWindow(hwnd);
      return FALSE;
      case WM_DESTROY:
		 systray_restore();
         PostQuitMessage(0);
      break;
	  case WM_USER:
		  if (LOWORD(lParam) == WM_LBUTTONUP || LOWORD(lParam) == WM_RBUTTONUP ||
					LOWORD(lParam) == WM_MBUTTONUP) {
			  POINT p;
              GetCursorPos(&p);
              SetForegroundWindow(hwnd);
              TrackPopupMenu(main_menu,TPM_RIGHTBUTTON,p.x,p.y,0,hwnd,NULL);
		  }
	  break;
   }
   return DefWindowProc(hwnd, Message, wParam, lParam);
}


void MakeModeList()
{
	int res;
	DEVMODE dm;
	ModeList = NULL;
	ModeList_Length=0;
	do {
		res = EnumDisplaySettings(NULL,ModeList_Length,&dm);
		if (res)
		{
			int index = ModeList_Length++;
			ModeList = (ModeType *) realloc(ModeList,sizeof(ModeType)*ModeList_Length);
			ModeList[index].xres = dm.dmPelsWidth;
			ModeList[index].yres = dm.dmPelsHeight;
			ModeList[index].bits = dm.dmBitsPerPel;
			ModeList[index].ref  = dm.dmDisplayFrequency;
		}
	} while (res);
}

int mycmp(const int *elem1, const int *elem2 )
{
	if (*elem1-*elem2)
		return (*elem1-*elem2);
	else return (*(elem1+1)-*(elem2+1));
}
int mycmp2(const int *elem1, const int *elem2 )
{
	return (*elem1-*elem2);
}

void MakeMenu(HMENU hmenu)
{
	int x, y;
	ModeType *p = ModeList;
	char str[128];
	num_color_depths = 0;
	for (x = 0; x < ModeList_Length; x ++, p++)
	{
		for (y = 0; y < num_color_depths && color_depths[y]; y ++)
			if (color_depths[y] == p->bits) break;
		if (y >= num_color_depths) 
		{
			num_color_depths++;
			color_depths = (int *) realloc(color_depths,num_color_depths * sizeof(int));
		}
		color_depths[y] = p->bits;
	}
	qsort(color_depths,num_color_depths,sizeof(int),mycmp2);
	for (y = 0; y < num_color_depths; y ++) 
	{
		wsprintf(str,"%3d bits/pixel",color_depths[y]);
		InsertMenu(hmenu,2+y,MF_BYPOSITION|MF_STRING,ID_BPPBASE+y,str);
	}
	num_resolutions = 0;
	p = ModeList;
	for (x = 0; x < ModeList_Length; x ++, p++)
	{
		for (y = 0; y < num_resolutions && resolutions[y*3]; y ++)
			if (resolutions[y*3] == p->xres && resolutions[y*3+1] == p->yres
				&& resolutions[y*3+2] == p->ref) break;
		if (y >= num_resolutions) 
		{
			num_resolutions++;
			resolutions = (int *) realloc(resolutions,num_resolutions * sizeof(int) * 3);
		}
		resolutions[y*3] = p->xres;
		resolutions[y*3+1] = p->yres;
		resolutions[y*3+2] = p->ref;
	}
	qsort(resolutions,num_resolutions,sizeof(int)*3,mycmp);
	for (x = 0; x < num_resolutions; x ++) 
	{
		if (resolutions[x*3+2])
			wsprintf(str,"%dx%-4d @ %3dhz",resolutions[x*3],resolutions[x*3+1],resolutions[x*3+2]);
		else
			wsprintf(str,"%dx%-4d",resolutions[x*3],resolutions[x*3+1]);
		InsertMenu(hmenu,3+x+num_color_depths,MF_BYPOSITION|MF_STRING,ID_MODEBASE+x,str);
	}
}

int SetVidMode(int xres, int yres, int bpp, int ref)
{
	DEVMODE dm;
	dm.dmPelsWidth = xres;
	dm.dmPelsHeight = yres;
	dm.dmBitsPerPel = bpp;
	dm.dmDisplayFrequency = ref;
	dm.dmFields = DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT|DM_DISPLAYFREQUENCY;
    return (ChangeDisplaySettings(&dm,CDS_UPDATEREGISTRY));
}

void setTip()
{
	char buf[129];
	if (resolutions[cur_res*3+2])
		wsprintf(buf,"%dx%d %dBPP @ %dhz - jRes v0.6",
		resolutions[cur_res*3],
		resolutions[cur_res*3+1],
		color_depths[cur_depth],
		resolutions[cur_res*3+2]);
	else
		wsprintf(buf,"%dx%d %dBPP - jRes v0.6",
		resolutions[cur_res*3],
		resolutions[cur_res*3+1],
		color_depths[cur_depth]);

	systray_minimize(buf);
}


void doVidModeErr(int m)
{
	switch (m) {
		case DISP_CHANGE_SUCCESSFUL: return;
		case DISP_CHANGE_RESTART:
			MessageBox(hMainWindow,"A restart appears to be required\n"
								   "for this video mode.","Error",MB_OK|MB_ICONEXCLAMATION);
		break;
		case DISP_CHANGE_FAILED:
			MessageBox(hMainWindow,"The graphics adapter couldn't\n"
								   "set this video mode.","Error",MB_OK|MB_ICONEXCLAMATION);
		break;
		case DISP_CHANGE_BADMODE:
			MessageBox(hMainWindow,"The graphics adapter doesn't\n"
								   "support this video mode.","Error",MB_OK|MB_ICONEXCLAMATION);
		break;
		case DISP_CHANGE_NOTUPDATED:
			MessageBox(hMainWindow,"Couldn't write settings to \n"
								   "registry.","Warning",MB_OK|MB_ICONEXCLAMATION);
		return;
		default:
			MessageBox(hMainWindow,"Could not set mode","Error",MB_OK|MB_ICONEXCLAMATION);
		break;
	}
}
