/*
** systray.c: jRes system tray icon management.
** Copyright (C) 1997-1998, Nullsoft Inc.
** Free for noncommercial use.
** http://www.nullsoft.com/
**/

#include <windows.h>
#include "resource.h"

#define SYSTRAY_ICON 566
#define SYSTRAY_WINDOW hMainWindow
extern HWND hMainWindow;
extern HINSTANCE hMainInstance;
BOOL systray_intray;

static BOOL systray_add(HWND hwnd, UINT uID, LPSTR lpszTip);
static BOOL systray_del(HWND hwnd, UINT uID);
static BOOL systray_mod(HWND hwnd, UINT uID, LPSTR lpszTip);
static HICON hhIcon;

void systray_minimize(char *tip) {
  if (!hhIcon) hhIcon = LoadIcon(hMainInstance,MAKEINTRESOURCE(IDI_ICON1));
  if (!systray_intray) {
		systray_add(SYSTRAY_WINDOW,SYSTRAY_ICON,tip);
		systray_intray = 1;
  } else {
    systray_mod(SYSTRAY_WINDOW,SYSTRAY_ICON,tip);
  }
}

void systray_restore() {
  if (systray_intray) {
		systray_del(SYSTRAY_WINDOW,SYSTRAY_ICON);
		SetForegroundWindow(SYSTRAY_WINDOW);
		systray_intray = 0;
		if (hhIcon) DestroyIcon(hhIcon);
		hhIcon = 0;
  }
}

static BOOL systray_add(HWND hwnd, UINT uID, LPSTR lpszTip) {
  NOTIFYICONDATA tnid;
  tnid.cbSize = sizeof(NOTIFYICONDATA);
  tnid.hWnd = hwnd;
  tnid.uID = uID;
  tnid.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
  tnid.uCallbackMessage = WM_USER;
  tnid.hIcon = hhIcon;
  strncpy(tnid.szTip,lpszTip,sizeof(tnid.szTip)-1);
  return (Shell_NotifyIcon(NIM_ADD, &tnid));
}

static BOOL systray_del(HWND hwnd, UINT uID) {
  NOTIFYICONDATA tnid;
  tnid.cbSize = sizeof(NOTIFYICONDATA);
  tnid.hWnd = hwnd;
  tnid.uID = uID;
  return(Shell_NotifyIcon(NIM_DELETE, &tnid));
}

static BOOL systray_mod(HWND hwnd, UINT uID, LPSTR lpszTip) {
  NOTIFYICONDATA tnid;
  tnid.cbSize = sizeof(NOTIFYICONDATA);
  tnid.hWnd = hwnd;
  tnid.uID = uID;
  tnid.uFlags = NIF_TIP;// | NIF_MESSAGE | NIF_ICON;
//  tnid.uCallbackMessage = WM_USER;
//  tnid.hIcon = hhIcon;
  strncpy(tnid.szTip,lpszTip,sizeof(tnid.szTip)-1);
  return (Shell_NotifyIcon(NIM_MODIFY, &tnid));
}
